import argparse
import copy
import unittest
from unittest import mock

from schema import SchemaError

import skelebot as sb

class TestRegistry(unittest.TestCase):

    registry = {
        "host": "docker.io",
        "port": 88,
        "user": "skelebot"
    }

    def test_addParsers(self):
        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")

        parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        subparsers = parser.add_subparsers(dest="job")
        subparsers = registry.addParsers(subparsers)

        self.assertNotEqual(subparsers.choices["publish"], None)

    @mock.patch('skelebot.components.registry.docker')
    def test_execute(self, mock_docker):
        mock_docker.build.return_value = 0

        config = sb.objects.config.Config(language="R")
        args = argparse.Namespace()

        registry = sb.components.registry.Registry(host="docker.io", port=88, user="skelebot")
        registry.execute(config, args)

        mock_docker.login.assert_called_with("docker.io")
        mock_docker.build.assert_called_with(config)
        mock_docker.push.assert_called_with(config, "docker.io", 88, "skelebot")

    def test_validate_valid(self):
        try:
            sb.components.registry.Registry.validate(self.registry)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

    def validate_error(self, attr, reset, expected):
        registry = copy.deepcopy(self.registry)
        registry[attr] = reset

        try:
            sb.components.registry.Registry.validate(registry)
        except SchemaError as error:
            self.assertEqual(error.code, "Registry '{attr}' must be a{expected}".format(attr=attr, expected=expected))

    def test_invalid(self):
        self.validate_error('host', 123, ' String')
        self.validate_error('port', "abc", 'n Integer')
        self.validate_error('user', 123, ' String')

if __name__ == '__main__':
    unittest.main()



"""README File Generator"""

import os

FILE_PATH = "{path}/README.md"
README = """# {name}
![Version](https://img.shields.io/badge/Version-{version}-brightgreen.svg)
![Documentation](https://img.shields.io/badge/Documentation-UNLINKED-red.svg)

{description}

---

## Contact
Project Maintainer: {maintainer} ({contact})"""

# Builds the README based on the config
def buildREADME(config):
    """Build the README.md file based on values from the Config object"""

    name = config.name
    desc = config.description
    version = config.version
    maintainer = config.maintainer
    contact = config.contact

    readme = README.format(name=name, description=desc, version=version, maintainer=maintainer,
                           contact=contact)

    readmeFile = open(FILE_PATH.format(path=os.getcwd()), "w")
    readmeFile.write(readme)
    readmeFile.close()



"""YAML Generator"""

import os
import copy
import yaml
from ...objects.config import Config

FILE_PATH = "{path}/skelebot.yaml"
VERSION_PATH = "{path}/VERSION"
ENV_FILE_PATH = "{path}/skelebot-{env}.yaml"

def loadConfig(env=None):
    """
    Attempt to load the skelebot.yaml file into a Config object along with all of the activated
    componenets
    """

    config = Config.load(readYaml(env))
    config.version = loadVersion()
    return config

def saveConfig(config):
    """Generate the skelebot.yaml and VERSION files with the values from the Config object"""

    saveVersion(config.version)
    config.version = None
    yml = yaml.dump(config.toDict(), default_flow_style=False)

    with open(FILE_PATH.format(path=os.getcwd()), "w") as file:
        file.write(yml)

def loadVersion():
    """Load the version number from the VERSION file"""

    version = None
    versionFile = VERSION_PATH.format(path=os.getcwd())
    if os.path.isfile(versionFile):
        with open(versionFile, 'r') as file:
            version = file.read().replace("\n", "")
    return version

def saveVersion(version):
    """Overwrite the version number in the VERSION file with a new version"""

    with open(VERSION_PATH.format(path=os.getcwd()), 'w') as file:
        file.write(version)

def readYaml(env=None):
    """Load the skelebot.yaml, with environment overrride if present, into the Config object"""

    yamlData = None
    cwd = os.getcwd()
    cfgFile = FILE_PATH.format(path=cwd)
    if os.path.isfile(cfgFile):
        with open(cfgFile, 'r') as stream:
            yamlData = yaml.load(stream, Loader=yaml.FullLoader)
            if (env is not None):
                envFile = ENV_FILE_PATH.format(path=cwd, env=env)
                if os.path.isfile(envFile):
                    with open(envFile, 'r') as stream:
                        overrideYaml = yaml.load(stream, Loader=yaml.FullLoader)
                        yamlData = override(yamlData, overrideYaml)
                else:
                    raise RuntimeError("Environment Not Found")

    return yamlData

def override(orig, over):
    """Override one dictionary with data from another dictionary"""

    merged = copy.deepcopy(orig)
    for k, v2 in over.items():
        merged[k] = copy.deepcopy(v2)
    return merged



import copy
import unittest

from schema import SchemaError

import skelebot as sb

class TestConfigValidate(unittest.TestCase):

    config = {
        'name': 'test',
        'description': 'test',
        'maintainer': 'test',
        'contact': 'test',
        'language': 'test',
        'baseImage': 'test',
        'primaryJob': 'test',
        'ephemeral': True,
        'dependencies': [1, 2],
        'ignores': [1, 2],
        'jobs': [1, 2],
        'ports': [1, 2],
        'components': {},
        'params': [1, 2],
        'commands': [],
    }

    def validate_error(self, attr, reset, expected):
        config = copy.deepcopy(self.config)
        config[attr] = reset

        try:
            sb.objects.config.Config.validate(config)
        except SchemaError as error:
            self.assertEqual(error.code, "'{attr}' must be a {expected}".format(attr=attr, expected=expected))

    def test_valid(self):

        try:
            sb.objects.config.Config.validate(self.config)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

    def test_invalid_mising(self):
        config = copy.deepcopy(self.config)
        del config['name']
        del config['language']

        try:
            sb.objects.config.Config.validate(config)
        except SchemaError as error:
            self.assertEqual(error.code, "Missing keys: 'language', 'name'")

    def test_invalid(self):
        self.validate_error('name', 123, 'String')
        self.validate_error('description', 123, 'String')
        self.validate_error('maintainer', 123, 'String')
        self.validate_error('contact', 123, 'String')
        self.validate_error('language', 123, 'String')
        self.validate_error('baseImage', 123, 'String')
        self.validate_error('primaryJob', 123, 'String')
        self.validate_error('ephemeral', 123, 'Boolean')
        self.validate_error('dependencies', 123, 'List')
        self.validate_error('ignores', 123, 'List')
        self.validate_error('jobs', 123, 'List')
        self.validate_error('ports', 123, 'List')
        self.validate_error('components', 123, 'Dictionary')
        self.validate_error('params', 123, 'List')
        self.validate_error('commands', 123, 'List')

if __name__ == '__main__':
    unittest.main()



"""Docker Exec Component"""

from ..objects.component import Activation, Component
from ..systems.execution import docker

class Dexec(Component):
    """
    Docker Exec Class

    Provides a command to access the project container via bash prompt with the option volume map
    the contents of the project into the container so that they can be modified as root
    """

    activation = Activation.PROJECT
    commands = ["exec"]

    def addParsers(self, subparsers):
        """
        SkeleParser Hook

        Adds a parser for the exec command that allows for bash access to the project container
        along with full project volume mapping support via an optional parameter flag
        """

        helpMessage = "Exec into the running Docker container"
        argHelp = "Volume map the working directory onto the container"
        parser = subparsers.add_parser("exec", help=helpMessage)
        parser.add_argument("-m", "--map", action="store_true", help=argHelp)
        return subparsers

    def execute(self, config, args):
        """
        Execution Hook

        Executes when the exec command is provided and runs the container with the /bin/bash
        command in order to provide access to the user
        """

        mappings = []
        if (args.map):
            mappings.append(".")

        docker.build(config)
        docker.run(config, "/bin/bash", "it", [], mappings, "exec")



"""Component Factory"""

import os
import sys
import importlib
from ..objects.component import Activation
from ..common import PLUGINS_HOME
from .plugin import Plugin
from .jupyter import Jupyter
from .kerberos import Kerberos
from .bump import Bump
from .prime import Prime
from .dexec import Dexec
from .artifactory import Artifactory
from .registry import Registry

class ComponentFactory():
    """
    Component Factory Class

    Maintains and builds all of the components in Skelebot including plugin components installed
    in the Skelebot Home folder
    """

    COMPONENTS = None

    def __init__(self):
        """Initializes the default list of components in Skelebot"""

        self.COMPONENTS = {
            Plugin.__name__.lower(): Plugin,
            Jupyter.__name__.lower(): Jupyter,
            Kerberos.__name__.lower(): Kerberos,
            Bump.__name__.lower(): Bump,
            Prime.__name__.lower(): Prime,
            Dexec.__name__.lower(): Dexec,
            Artifactory.__name__.lower(): Artifactory,
            Registry.__name__.lower(): Registry

        }

        # Add the plugin components to the master list
        pluginsHome = os.path.expanduser(PLUGINS_HOME)
        if (os.path.exists(pluginsHome)):
            sys.path.append(pluginsHome)
            for pluginName in os.listdir(pluginsHome):
                if (pluginName[0] != "_"):
                    module = importlib.import_module("{name}.{name}".format(name=pluginName))
                    plugin = getattr(module, pluginName[0].upper() + pluginName[1:])
                    self.COMPONENTS[pluginName.lower()] = plugin

    def buildComponents(self, activations=None, ignores=None):
        """Constructs a list of components based on which are active and which should be ignored"""

        activations = activations if activations is not None else [Activation.ALWAYS]
        ignores = ignores if ignores is not None else []
        components = []
        for component in list(self.COMPONENTS.values()):
            if (component.activation in activations) and (component.__name__ not in ignores):
                components.append(component())

        return components

    def buildComponent(self, name, data=None):
        """Builds a single component object based on it's name with the data provided"""

        return self.COMPONENTS[name].load(data) if (name in self.COMPONENTS) else None



import copy
import unittest

from schema import SchemaError

import skelebot as sb

class TestParamValidate(unittest.TestCase):

    param = {
        'name': 'test',
        'alt': 'test',
        'accepts': 'test',
        'default': 'test',
        'help': 'test',
        'choices': [1, 2]
    }

    def validate_error(self, attr, reset, expected):
        param = copy.deepcopy(self.param)
        param[attr] = reset

        try:
            sb.objects.param.Param.validate(param)
        except SchemaError as error:
            self.assertEqual(error.code, "Param '{attr}' must be a {expected}".format(attr=attr, expected=expected))

    def test_valid(self):

        try:
            sb.objects.param.Param.validate(self.param)
        except:
            self.fail("Validation Raised Exception Unexpectedly")

    def test_invalid_mising(self):
        param = copy.deepcopy(self.param)
        del param['name']

        try:
            sb.objects.param.Param.validate(param)
        except SchemaError as error:
            self.assertEqual(error.code, "Missing key: 'name'")

    def test_invalid(self):
        self.validate_error('name', 123, 'String')
        self.validate_error('alt', 123, 'String')
        self.validate_error('accepts', 123, 'String')
        self.validate_error('help', 123, 'String')
        self.validate_error('choices', 123, 'List')

if __name__ == '__main__':
    unittest.main()



"""Docker Execution"""

import os
from ...systems.generators import dockerfile
from ...systems.generators import dockerignore

LOGIN_CMD = "docker login {}"
BUILD_CMD = "docker build -t {image} ."
RUN_CMD = "docker run --name {image}-{jobName} --rm {params} {image} /bin/bash -c \"{command}\""
SAVE_CMD = "docker save -o {filename} {image}"
TAG_CMD = "docker tag {src} {image}:{version}"
PUSH_CMD = "docker push {image}:{version}"

def login(host=None):
    """Login to the given Docker Host"""

    host = host if host is not None else ""
    loginCMD = LOGIN_CMD.format(host)

    print(loginCMD)
    status = os.system(loginCMD)

    if (status != 0):
        raise Exception("Docker Login Failed")

def build(config):
    """Build the Docker Image after building the Dockerfile and .dockerignore from Config"""

    # Build Dockerfile, .dockerignore, and Docker Image
    dockerfile.buildDockerfile(config)
    dockerignore.buildDockerignore(config)
    status = os.system(BUILD_CMD.format(image=config.getImageName()))

    # Remove Files if ephemeral is set to True in Config
    if (config.ephemeral):
        os.remove("Dockerfile")
        os.remove(".dockerignore")

    # Raise an error if the build process failed
    if (status > 0):
        raise Exception("Docker Build Failed")

    return status

def run(config, command, mode, ports, mappings, task):
    """Run the Docker Container from the Image with the provided command"""

    params = "-{mode}".format(mode=mode)

    # Construct the port mappings
    if (ports):
        for port in ports:
            params += " -p {port}".format(port=port)

    # Construct the volume mappings
    if (mappings):
        for vmap in mappings:
            if ("~" in vmap):
                vmap = vmap.replace("~", os.path.expanduser("~"))

            if (":" in vmap):
                params += " -v {vmap}".format(vmap=vmap)
            else:
                params += " -v {pwd}/{vmap}:/app/{vmap}".format(pwd=os.getcwd(), vmap=vmap)

    # Construct the additional parameters from the components
    for component in config.components:
        addParams = component.addDockerRunParams()
        if (addParams is not None):
            params += " {params}".format(params=addParams)

    # Assuming the image was built without errors, run the container with the given command
    image = config.getImageName()
    runCMD = RUN_CMD.format(image=image, jobName=task, command=command, params=params, mode=mode)
    return os.system(runCMD)

def save(config, filename="image.img"):
    """Save the Image File to the disk"""

    return os.system(SAVE_CMD.format(image=config.getImageName(), filename=filename))

def push(config, host=None, port=None, user=None):
    """Tag with version and latest and push the project Image to the provided Docker Image Host"""

    imageName = config.getImageName()
    port = ":{port}".format(port=port) if port is not None else ""
    host = "{host}{port}/".format(host=host, port=port) if host is not None else ""
    user = "{user}/".format(user=user) if user is not None else ""
    image = "{host}{user}{name}".format(host=host, port=port, user=user, name=imageName)

    status = os.system(TAG_CMD.format(src=imageName, image=image, version=config.version))
    if (status == 0):
        status = os.system(TAG_CMD.format(src=imageName, image=image, version="latest"))
    if (status == 0):
        status = os.system(PUSH_CMD.format(image=image, version=config.version))
    if (status == 0):
        status = os.system(PUSH_CMD.format(image=image, version="latest"))

    if (status != 0):
        raise Exception("Docker Push Failed")



"""SkeleYaml Base Object"""

import copy

class SkeleYaml():
    """
    SkeleYaml Class

    This class acts as a base for any object that needs to be converted to a Dict for YAML file
    generation purposes
    """

    schema = None

    @classmethod
    def loadList(cls, config):
        """Iterates over a list of Dicts that represent SkeleYamls and loads them into a list"""

        objs = []
        if isinstance(config, list):
            for element in config:
                objs.append(cls.load(element))

        return objs

    @classmethod
    def load(cls, config):
        """Defines the default manner of loading a Component object from a config dict"""

        cls.validate(config)
        return cls(**config)

    @classmethod
    def validate(cls, config):
        """Executes the schema validation based on the defined Schema for the implemented class"""

        if cls.schema is not None:
            cls.schema.validate(config)

    def toDict(self):
        """Convert the object attributes into a Dict structure that is ready for YAML generation"""

        dct = copy.deepcopy(vars(self))
        removeList = []
        for key in dct.keys():
            if (dct[key] is None) or (dct[key] == []):
                removeList.append(key)
            elif (isinstance(dct[key], SkeleYaml)):
                dct[key] = dct[key].toDict()
            elif (isinstance(dct[key], list)):
                dctList = []
                for element in dct[key]:
                    dctList.append(element.toDict() if isinstance(element, SkeleYaml) else element)
                dct[key] = dctList

        for key in removeList:
            del dct[key]

        return dct



"""Artifactory Component"""

import os
import shutil
import artifactory
from requests.exceptions import MissingSchema
from schema import Schema, And, Optional
from ..objects.component import Activation, Component
from ..objects.skeleYaml import SkeleYaml

ERROR_ALREADY_PUSHED = """This artifact version has already been pushed.
Please bump the version before pushing (skelebot bump) or force push (-f)."""

def pushArtifact(artifactFile, user, token, file, url, force):
    """Pushes the given file to the url with the provided user/token auth"""

    # Error and exit if artifact already exists and we are not forcing an override
    try:
        if (not force) and (artifactory.ArtifactoryPath(url, auth=(user, token)).exists()):
            raise Exception(ERROR_ALREADY_PUSHED)
    except MissingSchema:
        pass

    # Rename artifact, deploy the renamed artifact, and then rename it back to original name
    print("Deploying {file} to {url}".format(file=file, url=url))
    path = artifactory.ArtifactoryPath(url, auth=(user, token))
    shutil.copyfile(artifactFile, file)
    try:
        path.deploy_file(file)
        os.remove(file)
    except:
        os.remove(file)
        raise

def pullArtifact(user, token, file, url):
    """Pulls the given file from the url with the provided user/token auth"""

    if (artifactory.ArtifactoryPath(url, auth=(user, token)).exists()):
        print("Pulling {file} from {url}".format(file=file, url=url))
        path = artifactory.ArtifactoryPath(url, auth=(user, token))
        with path.open() as fd:
            with open(file, "wb") as out:
                out.write(fd.read())
    else:
        print("Artifact Not Found: {url}".format(url=url))

class Artifact(SkeleYaml):
    """
    Artifact Class

    Object contained within a list of the artifactory configuration in order to specify the
    artifact files and artifact names
    """

    schema = Schema({
        'name': And(str, error='Artifact \'name\' must be a String'),
        'file': And(str, error='Artifact \'file\' must be a String'),
    }, ignore_extra_keys=True)

    name = None
    file = None

    def __init__(self, name, file):
        self.name = name
        self.file = file

class Artifactory(Component):
    """
    Artifactory Component Class

    Provides the ability to push and pull artifacts that are defined in the skelebot config
    file to and from Artifactory based on the project version number
    """

    activation = Activation.CONFIG
    commands = ["push", "pull"]

    schema = Schema({
        Optional('artifacts'): And(list, error='Artifactory \'artifacts\' must be a List'),
        'url': And(str, error='Artifactory \'url\' must be a String'),
        'repo': And(str, error='Artifactory \'repo\' must be a String'),
        'path': And(str, error='Artifactory \'path\' must be a String'),
    }, ignore_extra_keys=True)

    artifacts = None
    url = None
    repo = None
    path = None

    @classmethod
    def load(cls, config):
        """Instantiate the Artifact Class Object based on a config dict"""

        cls.validate(config)

        artifactDicts = config["artifacts"]
        artifacts = []
        for artifact in artifactDicts:
            newArtifact = Artifact.load(artifact)
            artifacts.append(newArtifact)

        return cls(artifacts, config["url"], config["repo"], config["path"])

    def __init__(self, artifacts=None, url=None, repo=None, path=None):
        """Instantiate the Artifact Class Object based on the provided parameters"""

        self.artifacts = artifacts if artifacts is not None else []
        self.url = url
        self.repo = repo
        self.path = path

    def addParsers(self, subparsers):
        """
        SkeleParser Hook

        Adds the parsers for push and pull commands to the SkeleParser to push artifacts
        with the project version and pull artifacts with the provided version number
        """

        artifactNames = []
        for artifact in self.artifacts:
            artifactNames.append(artifact.name)

        parser = subparsers.add_parser("push", help="Push an artifact to artifactory")
        parser.add_argument("artifact", choices=artifactNames)
        parser.add_argument("-u", "--user", help="Auth user for Artifactory")
        parser.add_argument("-t", "--token", help="Auth token for Artifactory")
        parser.add_argument("-f", "--force", action='store_true', help="Force the push")

        parser = subparsers.add_parser("pull", help="Pull an artifact from artifactory")
        parser.add_argument("artifact", choices=artifactNames)
        parser.add_argument("version", help="The version of the artifact to pull")
        parser.add_argument("-u", "--user", help="Auth user for Artifactory")
        parser.add_argument("-t", "--token", help="Auth token for Artifactory")

        return subparsers

    def execute(self, config, args):
        """
        Execution Hook

        Executes either the push or pull command depending on which one was provided to push
        rename the artifact and push it to Artifactory or pull down the given artifact version
        from Artifactory
        """

        artifactory.global_config = {
            self.url: {
                'username': None,
                'verify': True,
                'cert': None,
                'password': None
            }
        }

        # Get User and Token if not provided in args
        user = args.user
        token = args.token
        if (user is None):
            user = input("Please provide a valid Artifactory user: ")

        if (token is None):
            token = input("Please provide a valid Artifactory token: ")

        # Obtain the artifact that matches the provided name
        selectedArtifact = None
        for artifact in self.artifacts:
            if (artifact.name == args.artifact):
                selectedArtifact = artifact

        # Generate the local artifact file and the final Artifactory url path
        ext = selectedArtifact.file.split(".")[1]
        version = config.version if (args.job == "push") else args.version
        file = "{filename}_v{version}.{ext}"
        file = file.format(filename=selectedArtifact.name, version=version, ext=ext)
        url = "{url}/{repo}/{path}/{file}"
        url = url.format(url=self.url, repo=self.repo, path=self.path, file=file)

        # Push the artifact with the config version, or pull with the arg version
        if (args.job == "push"):
            pushArtifact(selectedArtifact.file, user, token, file, url, args.force)
        elif (args.job == "pull"):
            pullArtifact(user, token, file, url)



"""Scaffolder System"""

import os
from ...objects.config import Config
from ...components.componentFactory import ComponentFactory
from ...systems.generators import dockerfile, dockerignore, readme, yaml
from ...common import LANGUAGE_DEPENDENCIES
from .prompt import promptUser

def scaffold(existing=False):
    """Scaffold a new or existing project into a Skelebot project"""

    # Prompt for basic project information
    print("Scaffolding Skelebot Project")
    print("--:-" * 5, "-:--" * 5)
    name = promptUser("Enter a PROJECT NAME")
    description = promptUser("Enter a PROJECT DESCRIPTION")
    maintainer = promptUser("Enter a MAINTAINER NAME")
    contact = promptUser("Enter a CONTACT EMAIL")
    language = promptUser("Enter a LANGUAGE", options=list(LANGUAGE_DEPENDENCIES.keys()))

    # Iterate over components for additional prompts and add any components that are scaffolded
    components = []
    componentFactory = ComponentFactory()
    for component in componentFactory.buildComponents():
        component = component.scaffold()
        if (component is not None):
            if (isinstance(component, list)):
                components += component
            else:
                components.append(component)

    # Build the config object based on the user inputs
    config = Config(name=name, description=description, version="0.1.0", maintainer=maintainer,
                    contact=contact, language=language, primaryJob=None, ephemeral=False,
                    dependencies=LANGUAGE_DEPENDENCIES[language], components=components)

    # Confirm user input - allow them to back out before generating files
    print("--:-" * 5, "-:--" * 5)
    print("Setting up the", name, "Skelebot project in the current directory")
    print("(", os.getcwd(), ")")
    if (not promptUser("Confirm Skelebot Setup", boolean=True)):
        raise Exception("Aborting Scaffolding Process")

    print("--:-" * 5, "-:--" * 5)
    if (not existing):
        # Setting up the folder structure for the project
        print("Wiring up the skele-bones...")
        os.makedirs("config/", exist_ok=True)
        os.makedirs("data/", exist_ok=True)
        os.makedirs("models/", exist_ok=True)
        os.makedirs("notebooks/", exist_ok=True)
        os.makedirs("output/", exist_ok=True)
        os.makedirs("queries/", exist_ok=True)
        os.makedirs("src/jobs/", exist_ok=True)

        # Creating the files for the project
        print("Soldering the micro-computer to the skele-skull...")
        dockerfile.buildDockerfile(config)
        dockerignore.buildDockerignore(config)
        readme.buildREADME(config)

    # For existing projects only the skelebot.yaml file is generated
    yaml.saveConfig(config)
    print("Your Skelebot project is ready to go!")



"""Registry Component"""

from schema import Schema, And, Optional
from ..objects.component import Activation, Component
from ..systems.execution import docker

class Registry(Component):
    """
    Registry Class

    Provides the ability to define the registry that will be used when publishing the Docker image
    for the project, and provides the capability to publish by logging into the registry through
    the CLI.
    """

    activation = Activation.PROJECT
    commands = ["publish"]

    schema = Schema({
        Optional('host'): And(str, error='Registry \'host\' must be a String'),
        Optional('port'): And(int, error='Registry \'port\' must be an Integer'),
        Optional('user'): And(str, error='Registry \'user\' must be a String'),
    }, ignore_extra_keys=True)

    host = None
    port = None
    user = None

    def __init__(self, host=None, port=None, user=None):
        """Instantiate the Registry Class Object based on the provided parameters"""
        self.host = host
        self.port = port
        self.user = user

    def addParsers(self, subparsers):
        """
        SkeleParser Hook

        Adds a parser for the publish command to allow the user to login and push the project's
        Docker image to the defined registry, or Docker Hub, if one is not defined.
        """

        helpMessage = "Publish your versioned Docker Image to the registry"
        subparsers.add_parser("publish", help=helpMessage)

        return subparsers

    def execute(self, config, args):
        """
        Execution Hook

        Executes when the publish command is provided and prompts for username and password before
        building the project's Docker Image and pushing it to the defined registry.
        """

        docker.login(self.host)
        docker.build(config)
        docker.push(config, self.host, self.port, self.user)



import os
import unittest
from unittest import mock
from argparse import Namespace

import skelebot as sb

class TestDocker(unittest.TestCase):
    path = ""

    # Get the path to the current working directory before we mock the function to do so
    def setUp(self):
        self.path = os.getcwd()

    @mock.patch('os.system')
    def test_login(self, mock_system):
        mock_system.return_value = 0

        sb.systems.execution.docker.login()
        mock_system.assert_called_once_with("docker login ")

    @mock.patch('os.system')
    def test_login_host(self, mock_system):
        mock_system.return_value = 0

        sb.systems.execution.docker.login("docker.io")
        mock_system.assert_called_once_with("docker login docker.io")

    @mock.patch('os.system')
    def test_login_error(self, mock_system):
        mock_system.return_value = 1

        with self.assertRaisesRegex(Exception, "Docker Login Failed"):
            sb.systems.execution.docker.login()

    @mock.patch('os.path.expanduser')
    @mock.patch('os.system')
    @mock.patch('os.getcwd')
    def test_push(self, mock_getcwd, mock_system, mock_expanduser):
        host = "docker.io"
        port = 8888
        user = "skelebot"
        folderPath = "{path}/test/files".format(path=self.path)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        mock_system.return_value = 0

        config = sb.systems.generators.yaml.loadConfig()
        sb.systems.execution.docker.push(config, host, port, user)

        mock_system.assert_any_call("docker tag test docker.io:8888/skelebot/test:6.6.6")
        mock_system.assert_any_call("docker tag test docker.io:8888/skelebot/test:latest")
        mock_system.assert_any_call("docker push docker.io:8888/skelebot/test:6.6.6")
        mock_system.assert_any_call("docker push docker.io:8888/skelebot/test:latest")

    @mock.patch('os.path.expanduser')
    @mock.patch('os.system')
    @mock.patch('os.getcwd')
    def test_push_error(self, mock_getcwd, mock_system, mock_expanduser):
        host = "docker.io"
        port = 8888
        user = "skelebot"
        folderPath = "{path}/test/files".format(path=self.path)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        mock_system.return_value = 1

        config = sb.systems.generators.yaml.loadConfig()

        with self.assertRaisesRegex(Exception, "Docker Push Failed"):
            sb.systems.execution.docker.push(config, host, port, user)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.remove')
    @mock.patch('os.system')
    @mock.patch('os.getcwd')
    def test_build_ephemeral(self, mock_getcwd, mock_system, mock_remove, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        mock_system.return_value = 0

        config = sb.systems.generators.yaml.loadConfig()
        config.ephemeral = True

        sb.systems.execution.docker.build(config)
        mock_system.assert_called_once_with("docker build -t test .")
        mock_remove.assert_any_call("Dockerfile")
        mock_remove.assert_any_call(".dockerignore")

    @mock.patch('os.path.expanduser')
    @mock.patch('os.system')
    @mock.patch('os.getcwd')
    def test_build_non_ephemeral(self, mock_getcwd, mock_system, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        mock_system.return_value = 0

        config = sb.systems.generators.yaml.loadConfig()

        sb.systems.execution.docker.build(config)
        mock_system.assert_called_once_with("docker build -t test .")

    @mock.patch('os.path.expanduser')
    @mock.patch('os.system')
    @mock.patch('os.getcwd')
    def test_build_error(self, mock_getcwd, mock_system, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath
        mock_system.return_value = 1

        config = sb.systems.generators.yaml.loadConfig()

        with self.assertRaisesRegex(Exception, "Docker Build Failed"):
            sb.systems.execution.docker.build(config)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.system')
    @mock.patch('os.getcwd')
    def test_run(self, mock_getcwd, mock_system, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        args = Namespace(version='0.1')

        homePath = "{path}/test/plugins".format(path=self.path)
        mock_expanduser.return_value = homePath
        mock_getcwd.return_value = folderPath
        mock_system.return_value = 0

        config = sb.systems.generators.yaml.loadConfig()
        config.ports = ["1127:1127"]
        job = config.jobs[0]
        job.mappings = ["data/", "/test/output/:/app/output/", "~/temp/:/app/temp/"]
        command = sb.systems.execution.commandBuilder.build(config, job, args)

        expected = "docker run --name test-build --rm -i -p 1127:1127 -v {cwd}/data/:/app/data/ -v /test/output/:/app/output/ -v {path}/temp/:/app/temp/ test /bin/bash -c \"bash build.sh 0.1 --env local --log info\"".format(cwd=folderPath, path=homePath)
        sb.systems.execution.docker.run(config, command, job.mode, config.ports, job.mappings, job.name)
        mock_system.assert_called_once_with(expected)

    @mock.patch('os.system')
    def test_save(self, mock_system):
        mock_system.return_value = 0

        sb.systems.execution.docker.save(sb.objects.config.Config(name="Test Project"), "test.img")
        mock_system.assert_called_once_with("docker save -o test.img test-project")

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from unittest import mock
from argparse import Namespace

import skelebot as sb

class TestCommandBuilder(unittest.TestCase):
    path = ""

    # Get the path to the current working directory before we mock the function to do so
    def setUp(self):
        self.path = os.getcwd()

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_build(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        args = Namespace(version='0.1', test=True)

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath

        config = sb.systems.generators.yaml.loadConfig()
        job = config.jobs[0]
        param = sb.objects.param.Param("test", "t", accepts="boolean")
        job.params.append(param)

        expected = "bash build.sh 0.1 --env local --test --log info"
        command = sb.systems.execution.commandBuilder.build(config, job, args)
        self.assertEqual(command, expected)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_build_no_boolean(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        args = Namespace(version='1.1')

        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath

        config = sb.systems.generators.yaml.loadConfig()
        job = config.jobs[0]
        param = sb.objects.param.Param("test", "t", accepts="boolean")
        job.params.append(param)

        expected = "bash build.sh 1.1 --env local --log info"
        command = sb.systems.execution.commandBuilder.build(config, job, args)
        self.assertEqual(command, expected)

    @mock.patch('os.path.expanduser')
    @mock.patch('os.getcwd')
    def test_direct_command(self, mock_getcwd, mock_expanduser):
        folderPath = "{path}/test/files".format(path=self.path)
        mock_expanduser.return_value = "{path}/test/plugins".format(path=self.path)
        mock_getcwd.return_value = folderPath

        config = sb.systems.generators.yaml.loadConfig()
        job = config.jobs[1]

        expected = "echo Hello"
        command = sb.systems.execution.commandBuilder.build(config, job, None)
        self.assertEqual(command, expected)

if __name__ == '__main__':
    unittest.main()
